#include "display.h"
#include "vrp_pp.h"
#include <math.h>
#include <freeglut/freeglut.h>

DISPLAY_OBJECT::DISPLAY_OBJECT(class VRP_PP_PROBLEM *_PROBLEM, class VRP_PP_PLAN *_SOLUTION)
{
    this->PROBLEM = _PROBLEM;
    this->SOLUTION = _SOLUTION;

    this->xmax = -1000000;
    this->xmin = 1000000;
    this->ymin = 1000000;
    this->ymax = -1000000;

    if( _PROBLEM != NULL )
    {
        int N = this->PROBLEM->get_node_ctr();
        class VRP_PP_NODE TMP_NODE;
        for(int i=0 ; i < N ; i++ )
        {
            TMP_NODE = this->PROBLEM->get_node_by_index(i);
            if( this->xmax < TMP_NODE.longitude )
                this->xmax = TMP_NODE.longitude;
            if( this->xmin > TMP_NODE.longitude )
                this->xmin = TMP_NODE.longitude;
            if( this->ymax < TMP_NODE.lattitude )
                this->ymax = TMP_NODE.lattitude;
            if( this->ymin > TMP_NODE.lattitude )
                this->ymin = TMP_NODE.lattitude;

        }
    }

    this->x_factor = 1.0;
    this->y_factor = 1.0;

    if( ( xmax - xmin ) > (ymax - ymin) )
    {
        this->y_factor = (ymax - ymin) / ( xmax - xmin );
    }
    if( ( xmax - xmin ) < (ymax - ymin) )
    {
        this->x_factor = (xmax - xmin) / ( ymax - ymin );
    }
}

double DISPLAY_OBJECT::get_x(double x)
{
    double ret_val;

    if( (xmax - xmin) > 0 )
    {
        ret_val = -1.0 + 0.05 + 2 * 0.9 *( x - xmin ) / ( xmax - xmin ) ;
    }
    else
    {
        ret_val = 0;
    }

    return( ret_val * this->x_factor);
}

double DISPLAY_OBJECT::get_y(double y)
{
    double ret_val;

    if( (ymax - ymin) > 0 )
    {
        ret_val = -1.0 + +0.05 + 2 * 0.9 * ( y - ymin ) / ( ymax - ymin ) ;
    }
    else
    {
        ret_val = 0;
    }

    return( ret_val * this->y_factor);
}

DISPLAY_OBJECT::~DISPLAY_OBJECT()
{

}

void DISPLAY_OBJECT::print_nodes(void)
{
    if( this->PROBLEM != nullptr )
    {
        char S[256];
        int NODES = this->PROBLEM->get_node_ctr();
        class VRP_PP_NODE TmpNode;

        for( int i=0 ; i < NODES ; i++ )
        {
            // fetch the node from the problem object
            TmpNode = this->PROBLEM->get_node_by_index(i);
            // draw it
            double r = 0.020;
            int num_segments = 1000;

            glBegin(GL_LINE_LOOP);
            //glColor4f(0.0f, 1.0f, 0.0f, 0.0f);
            for(int ii = 0; ii < num_segments; ii++)
            {
                float theta = 2.0f * 3.1415926f * float(ii) / float(num_segments);//get the current angle

                float x = r * cosf(theta);//calculate the x component
                float y = r * sinf(theta);//calculate the y component

                glVertex2f(this->get_x(TmpNode.longitude), this->get_y(TmpNode.lattitude));
                glVertex2f(x + this->get_x(TmpNode.longitude), y + this->get_y(TmpNode.lattitude));//output vertex


            }
            glEnd();
            r = 0.015;
            glBegin(GL_LINE_LOOP);
            glColor4f(0.0f, 0.0f, 0.0f, 0.0f);
            for(int ii = 0; ii < num_segments; ii++)
            {
                float theta = 2.0f * 3.1415926f * float(ii) / float(num_segments);//get the current angle

                float x = r * cosf(theta);//calculate the x component
                float y = r * sinf(theta);//calculate the y component

                glVertex2f(this->get_x(TmpNode.longitude), this->get_y(TmpNode.lattitude));
                glVertex2f(x + this->get_x(TmpNode.longitude), y + this->get_y(TmpNode.lattitude));//output vertex


            }
            glEnd();
            glColor4f(0.0f, 1.0f, 0.0f, 0.0f);
            // print the node label

            glRasterPos2d(this->get_x(TmpNode.longitude)-0.01, this->get_y(TmpNode.lattitude)-0.01);
            sprintf(S,"%d",TmpNode.id);
            unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);

            glutBitmapString(GLUT_BITMAP_HELVETICA_12, ubuf);
            glColor4f(0.0f, 1.0f, 0.0f, 0.0f);
        }
    }
}

void DISPLAY_OBJECT::print_depots(void)
{
    if( this->PROBLEM != nullptr )
    {
        char S[256];
        int N_D = this->PROBLEM->get_depot_ctr();
        class VRP_PP_NODE TmpNode;
        class VRP_PP_REQUEST TmpDepot;

        glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
        for( int d=0 ; d < N_D ; d++)
        {
            // fetch the node from the problem object
            TmpDepot = this->PROBLEM->get_depot_by_index(d);
            TmpNode = this->PROBLEM->get_node_by_index(TmpDepot.id_node);

            // draw it
            glRectf(this->get_x(TmpNode.longitude)-0.02, this->get_y(TmpNode.lattitude)-0.02, this->get_x(TmpNode.longitude)+0.02, this->get_y(TmpNode.lattitude)+0.02);

        glColor4f(0.0f, 0.0f, 0.0f, 0.0f);
        for( int d=0 ; d < N_D ; d++)
        {
            // fetch the node from the problem object
            TmpDepot = this->PROBLEM->get_depot_by_index(d);
            TmpNode = this->PROBLEM->get_node_by_index(TmpDepot.id_node);

            // draw it
            glRectf(this->get_x(TmpNode.longitude)-0.015, this->get_y(TmpNode.lattitude)-0.015, this->get_x(TmpNode.longitude)+0.015, this->get_y(TmpNode.lattitude)+0.015);
        }
        glColor4f(1.0f, 1.0f, 1.0f, 0.0f);

        // print the node label

        glRasterPos2d(this->get_x(TmpNode.longitude)-0.01, this->get_y(TmpNode.lattitude)-0.01);
        sprintf(S,"%d",TmpNode.id);
        unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);

        glutBitmapString(GLUT_BITMAP_HELVETICA_12, ubuf);
        glColor4f(0.0f, 1.0f, 0.0f, 0.0f);
        }
    }
}

void DISPLAY_OBJECT::print_arcs(void)
{
    if( this->PROBLEM != nullptr )
    {
        char S[256];
        int N_A = this->PROBLEM->get_arc_ctr();
        class VRP_PP_ARC TmpArc;
        class VRP_PP_NODE NodeOrigin, NodeDestination;

        for( int a=0 ; a < N_A ; a++)
        {
            // fetch the arc from the problem object
            TmpArc = this->PROBLEM->get_arc_by_index(a);

            // fetch the two associated nodes
            NodeOrigin = this->PROBLEM->get_node_by_index(TmpArc.origin);
            NodeDestination = this->PROBLEM->get_node_by_index(TmpArc.dest);

            // draw the arc
            glColor4f(0.8f, 0.8f, 0.8f, 0.8f);
            glEnable(GL_LINE_SMOOTH);
            glLineWidth(0.01);
            glBegin(GL_LINES);
                glVertex2f(this->get_x(NodeOrigin.longitude), this->get_y(NodeOrigin.lattitude));
                glVertex2f(this->get_x(NodeDestination.longitude), this->get_y(NodeDestination.lattitude));
            glEnd();
            glLineWidth(1);
            glColor4f(0.0f, 1.0f, 0.0f, 0.0f);
            //cout << "a=" << a << ": " << this->get_x(NodeOrigin.longitude) <<  endl;
         }
    }
}

void DISPLAY_OBJECT::print_routes(void)
{
    if( this->SOLUTION != nullptr )
    {
        char S[256];
        class VRP_PP_OPERATION *Ptr;
        class VRP_PP_ROUTE *MyRoute;
        class VRP_PP_NODE Orig, Dest;

        float RED;
        float BLUE;
        float GREEN;
        float RED_OFFSET;
        float BLUE_OFFSET;
        float GREEN_OFFSET;
        for( int r=0 ; r < this->SOLUTION->get_route_ctr() ; r++ )
        {
            RED_OFFSET = 0.0;
            GREEN_OFFSET = 0.0;
            BLUE_OFFSET = 0.0;
            RED=0.0;
            GREEN=0.0;
            BLUE=0.0;
            if( (r % 3) == 0 )
            {
                RED = RED_OFFSET + 1.0 / (double)(1 + (double)(r) / (double)(this->SOLUTION->get_route_ctr()) );
                if( RED > 1.0)
                    RED = RED_OFFSET;
            }
            if( (r % 3) == 1 )
            {
                GREEN = GREEN_OFFSET + 1.0 / (double)(1 + (double)(r) / (double)(this->SOLUTION->get_route_ctr()) );
                if( GREEN > 1.0)
                    GREEN = GREEN_OFFSET;
            }
            if( (r % 3) == 2 )
            {
                BLUE = BLUE_OFFSET + 1.0 / (double)(1 + (double)(r) / (double)(this->SOLUTION->get_route_ctr()) );
                if( BLUE > 1.0)
                    BLUE = BLUE_OFFSET;

            }

            MyRoute = this->SOLUTION->get_route_by_index(r);
            Ptr = MyRoute->start;
            while( Ptr->next != nullptr )
            {
                Orig = this->PROBLEM->get_node_by_index(Ptr->id_node);
                Dest = this->PROBLEM->get_node_by_index(Ptr->next->id_node);

                glColor4f((RED), (GREEN), (BLUE), 0.0f);

                glEnable(GL_LINE_SMOOTH);
                glLineWidth(2);
                glBegin(GL_LINES);
                    glVertex2f(this->get_x(Orig.longitude), this->get_y(Orig.lattitude));
                    glVertex2f(this->get_x(Dest.longitude), this->get_y(Dest.lattitude));
                glEnd();
                glLineWidth(1);
                glColor4f(0.0f, 1.0f, 0.0f, 0.0f);

                Ptr = Ptr->next;
            }

            Ptr = MyRoute->start;
            while( Ptr->next != nullptr )
            {
                Orig = this->PROBLEM->get_node_by_index(Ptr->id_node);
                glColor4f(RED, GREEN, BLUE, 0.0f);
                if( Ptr != MyRoute->start )
                {
                    // draw it
                    double r = 0.020;
                    int num_segments = 1000;

                    glBegin(GL_LINE_LOOP);
                    for(int ii = 0; ii < num_segments; ii++)
                    {
                        float theta = 2.0f * 3.1415926f * float(ii) / float(num_segments);//get the current angle

                        float x = r * cosf(theta);//calculate the x component
                        float y = r * sinf(theta);//calculate the y component

                        glVertex2f(this->get_x(Orig.longitude), this->get_y(Orig.lattitude));
                        glVertex2f(x + this->get_x(Orig.longitude), y + this->get_y(Orig.lattitude));//output vertex

                    }
                    glEnd();
                    r = 0.015;
                    glBegin(GL_LINE_LOOP);
                    glColor4f(0.0f, 0.0f, 0.0f, 0.0f);
                    for(int ii = 0; ii < num_segments; ii++)
                    {
                        float theta = 2.0f * 3.1415926f * float(ii) / float(num_segments);//get the current angle

                        float x = r * cosf(theta);//calculate the x component
                        float y = r * sinf(theta);//calculate the y component

                        glVertex2f(this->get_x(Orig.longitude), this->get_y(Orig.lattitude));
                        glVertex2f(x + this->get_x(Orig.longitude), y + this->get_y(Orig.lattitude));//output vertex


                    }
                    glEnd();
                    glColor4f(RED, GREEN, BLUE, 0.0f);
                    // print the node label
                    glColor4f(1.0f, 1.0f, 1.0f, 0.0f);
                    glRasterPos2d(this->get_x(Orig.longitude)-0.01, this->get_y(Orig.lattitude)-0.01);
                    sprintf(S,"%d",Orig.id);
                    unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);

                    glutBitmapString(GLUT_BITMAP_HELVETICA_12, ubuf);
                    glColor4f(0.0f, 1.0f, 0.0f, 0.0f);

                }

                Ptr = Ptr->next;
            }
            glColor4f(0.0f, 1.0f, 0.0f, 0.0f);
        }
    }
}

void DISPLAY_OBJECT::print_route_agenda(void)
{
    if( this->SOLUTION != nullptr )
    {
        char S[256];
        class VRP_PP_ROUTE *MyRoute;

        float RED;
        float BLUE;
        float GREEN;
        float RED_OFFSET;
        float BLUE_OFFSET;
        float GREEN_OFFSET;
        for( int r=0 ; r < this->SOLUTION->get_route_ctr() ; r++ )
        {
            RED_OFFSET = 0.0;
            GREEN_OFFSET = 0.0;
            BLUE_OFFSET = 0.0;
            RED=0.0;
            GREEN=0.0;
            BLUE=0.0;
            if( (r % 3) == 0 )
            {
                RED = RED_OFFSET + 1.0 / (double)(1 + (double)(r) / (double)(this->SOLUTION->get_route_ctr()) );
                if( RED > 1.0)
                    RED = RED_OFFSET;
            }
            if( (r % 3) == 1 )
            {
                GREEN = GREEN_OFFSET + 1.0 / (double)(1 + (double)(r) / (double)(this->SOLUTION->get_route_ctr()) );
                if( GREEN > 1.0)
                    GREEN = GREEN_OFFSET;
            }
            if( (r % 3) == 2 )
            {
                BLUE = BLUE_OFFSET + 1.0 / (double)(1 + (double)(r) / (double)(this->SOLUTION->get_route_ctr()) );
                if( BLUE > 1.0)
                    BLUE = BLUE_OFFSET;

            }

            MyRoute = this->SOLUTION->get_route_by_index(r);

            // set the color of the route
            glColor4f((RED), (GREEN), (BLUE), 0.0f);

            // print the route label
            glRasterPos2d(0.9, -0.9 + 0.05*this->SOLUTION->get_route_ctr() - 0.05*r);
            sprintf(S,"%d",MyRoute->route_id);
            unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);

            glutBitmapString(GLUT_BITMAP_HELVETICA_12, ubuf);
            glColor4f(0.0f, 1.0f, 0.0f, 0.0f);

        }
    }
}
