#include <iostream>
#include <fstream>
#include <cmath>
#include <cstring>
#include <stdlib.h>
#include <ctime>

// needed to make vrp_pp objects known to the compiler
#include "vrp_pp.h"

// needed to inform the compiler which external software can be expected to be installed
#include "vrp_pp_extern.h"

//#include <GL/gl.h>
#include <freeglut/freeglut.h>
//#include "bussim_global.h"

#include "display.h"

using namespace std;

class DISPLAY_OBJECT *DISP;

void set_simulation_window(void)
{
    glutInitWindowSize(1000, 1000);
    glutCreateWindow("VRP_PP_VIEWER 1.1");

    glMatrixMode(GL_MODELVIEW);
        glLoadIdentity();
        glOrtho(-1, 1, -1, 1, -1, 1);

    //glMatrixMode(GL_MODELVIEW);
    //glLoadIdentity();

}

void tasten( unsigned char key, int x, int y )
{
    // here we receive and process the callback information from the keyboard
    switch( key ) {
    case 'q' :
        exit( EXIT_SUCCESS );
    break;
    case 'a' :
        DISP->DISPLAY_ARCS++;
        DISP->DISPLAY_ARCS = DISP->DISPLAY_ARCS % 2;
    break;
    case 'r' :
        DISP->DISPLAY_ROUTES++;
        DISP->DISPLAY_ROUTES = DISP->DISPLAY_ROUTES % 2;
    break;
    case 'n' :
        DISP->DISPLAY_NODES++;
        DISP->DISPLAY_NODES = DISP->DISPLAY_NODES % 2;
    break;
    }
    glutPostRedisplay();
}

// this is the main screen update method
void display(void)
{
        // specify and set the title of the simulation window
        char WindowTitle[256];
        sprintf(WindowTitle,"VRP_PP_VIEWER 1.00 %d longitudes: %.2f-%.2f lattitudes: %.2f-%.2f",rand() % 100, DISP->xmin, DISP->xmax, DISP->ymin, DISP->ymax);

        // update displayed information
        glutSetWindowTitle(WindowTitle);

       //Clear all pixels
        glClear(GL_COLOR_BUFFER_BIT);

        //load the windows identity
        // glLoadName(12);

        // this is the global color (used e.g. for the text)
        glColor4f(0.0f,1.0f,0.0f,0.0f);

        // print the arcs
        if( DISP->DISPLAY_ARCS > 0 )
            DISP->print_arcs();

        // print the routes
        if( DISP->DISPLAY_ROUTES > 0 )
            DISP->print_routes();

        // print the network nodes
        if( DISP->DISPLAY_NODES > 0 )
            DISP->print_nodes();

        // print the depot nodes
        DISP->print_depots();

        // print route agenda
        DISP->print_route_agenda();

        // update the graphic screen
        glFlush();

}

void set_callback_funcs(void)
{
    glutDisplayFunc(display);
    glutKeyboardFunc(tasten);
    //glutTimerFunc(10, time_loop, 3 /* stupid value to pass to func. */);
    //glutMenuStatusFunc(update_line_list_menue);
}


int main(int argc, char** argv)
{
    // try to read existing command line options and store them in the configuration object
	struct VRP_PP_CONFIG CONFIG;
    for (int k = 0; k < argc; k++)
	{
		if (strcmp(argv[k], "-p") == 0)
		{
			// we are talking about the xml-scenario file
			// go to the submitted value itself
			k++;
			// save the submitted value
			CONFIG.FILE = argv[k];
		}
		if (strcmp(argv[k], "-s") == 0)
		{
			// we are talking about the xml-scenario file
			// go to the submitted value itself
			k++;
			// save the submitted value
			CONFIG.PLANFILE = argv[k];
		}
	}

	// set the additional global algorithm parameters
        CONFIG.weigthType = VRP_PP_ARC_WEIGHT::GEO;     // specify the used arc weight

    // read problem file and create problem object
    class VRP_PP_PROBLEM prob(CONFIG.FILE);

    // read plan file and create plan object;
    class VRP_PP_PLAN plan(CONFIG.PLANFILE);

    // define the display-object

    DISP = new class DISPLAY_OBJECT(&(prob),&(plan));

    // create the graphic screen
    glutInit(&argc, argv);
    set_simulation_window();

    // these are the callback functions
    set_callback_funcs();

    glClearColor(0.0, 0.0, 0.0, 0.0);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    // Start the main loop
    glutMainLoop();

    return 0;
}
